#
# spec file for package gasket-driver
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


#===
# Packaging Notes:
#
# For information on building KMPs see:
# https://documentation.suse.com/sbp/all/single-html/SBP-KMP-Manual-SLE12SP2
# https://en.opensuse.org/Kernel_Module_Packages
#
# For information on `sysusers` macros used to create the `apex` group see:
# https://en.opensuse.org/openSUSE:Packaging_guidelines#Users_and_Groups
#---

# The following directive ensures the package will build correctly on OBS if it
# builds locally without problems. See:
# https://en.opensuse.org/openSUSE:Build_Service_Tips_and_Tricks#Permission_denied_errors
#
# norootforbuild

# Upstream doesn't really do proper releases, so track the main branch
# HEAD's commit hash as well.
%define git_version 5815ee3

Name:           gasket-driver
Version:        1.0.18_%{git_version}
Release:        3.14.4.1.sr20250601
Summary:        The Coral Gasket Driver allows usage of the Coral EdgeTPU on Linux systems
License:        GPL-2.0-only
Group:          System/Kernel
URL:            https://github.com/google/gasket-driver
Source0:        %{name}-%{version}.tar.xz
Source1:        group.conf
Source2:        preamble
Source3:        %{name}-rpmlintrc
BuildRequires:  modutils kernel-syms kmod-compat suse-kernel-rpm-scriptlets bash-sh kernel-syms-longterm
BuildRequires:  pesign-obs-integration
BuildRequires:  sysuser-tools
Requires:       %{name}-kmp
%sysusers_requires

# The dma-buf symbols used by this driver were moved into their own `DMA_BUF`
# module namespace in kernel 5.16. Upstream's current attempt at fixing this
# doesn't work with GCC versions that don't support the `__has_include` directive.
#
# Affects:
# - Leap 15.6
# - Leap 15.5
#
# See:
# https://github.com/google/gasket-driver/pull/10
# https://github.com/google/gasket-driver/pull/16
# https://github.com/google/gasket-driver/issues/17
#
# PATCH-FIX-OPENSUSE fix-for-backported-dma-buf-ns.patch gh#google/gasket-driver!10
Patch0:         fix-for-backported-dma-buf-ns.patch

# `no_llseek` was set to null in kernel versions >= 6.0, and then completely removed in kernel
# versions >= 6.12 resulting in broken builds.
#
# PATCH-FIX-OPENSUSE fix-for-no_llseek.patch
Patch1:         fix-for-no_llseek.patch

# In kernel 6.13 the MODULE_IMPORT_NS() function has been changed to accept string literals
# instead of macro expansions.
#
# Affects:
# - Tumbleweed
#
# See:
# https://github.com/google/gasket-driver/pull/40
# https://github.com/google/gasket-driver/issues/39
#
# PATCH-FIX-OPENSUSE  fix-for-module-import-ns.patch gh#google/gasket-driver!40
Patch2:         fix-for-module-import-ns.patch

# This directive instructs the build service to temporarily save the project's
# certificate as %%_sourcedir/_projectcert.crt. See:
# https://github.com/openSUSE/pesign-obs-integration
# https://documentation.suse.com/sbp/all/html/SBP-KMP-Manual/index.html#sec-signing-module-object
# https://documentation.suse.com/sbp/all/html/SBP-KMP-Manual/index.html#sec-appendix-a1
#
# needssslcertforbuild
#
# Having included the above directive, the below line will
# cause the "ueficert" package to get built.
%kernel_module_package -p %_sourcedir/preamble

%description
The Coral Gasket Driver allows usage of the Coral EdgeTPU on Linux systems.
The driver contains two modules:
- Gasket (Google ASIC Software, Kernel Extensions, and Tools) is a top level driver
  for lightweight communication with Google ASICs.
- Apex refers to the EdgeTPU v1.

%package KMP
Summary:        Gasket Driver kernel modules
Group:          System/Kernel

%description KMP
The Linux Kernel Module Package for the Coral Gasket Driver.

%prep
%setup -q
mkdir -p obj

# Apply the patch for `DMA_BUF` module namespace backports.
%if 0%{?sle_version} == 150500 || 0%{?sle_version} == 150600
%patch -P 0 -p1
%endif

# Apply `fix-for-no_llseek.patch`.
%patch -P 1 -p1

# Apply `fix-for-module-import-ns.patch`.
%if 0%{?suse_version} > 1600
%patch -P 2 -p1
%endif

%build
# Build the kernel modules.
for flavor in %flavors_to_build; do
       rm -rf obj/$flavor
       cp -r src obj/$flavor
       make -C %{kernel_source $flavor} modules M=$PWD/obj/$flavor
done
# Generate content to be used by the `%%pre` scriptlet.
%sysusers_generate_pre %{SOURCE1} group group.conf

# Execute the system group scriptlet generated at build time.
%pre -f group.pre

%install
# Install the kernel modules.
export INSTALL_MOD_PATH=%{buildroot}
export INSTALL_MOD_DIR=updates
for flavor in %flavors_to_build; do
       make -C %{kernel_source $flavor} modules_install M=$PWD/obj/$flavor
done
# Install the system group used by the driver.
mkdir -p %{buildroot}%{_sysusersdir}
install -m 0644 %{SOURCE1} -D %{buildroot}%{_sysusersdir}/group-apex.conf
# Install the udev rules defined in the module source.
install -D -m 644 debian/gasket-dkms.udev %buildroot%{_udevrulesdir}/70-apex.rules

# These env vars are used by the `pesign-obs-integration` package when signing
# the modules for Secure Boot.
export BRP_PESIGN_FILES='*.ko'
export BRP_PESIGN_COMPRESS_MODULE="xz"

%files
%license LICENSE
%doc README.md CONTRIBUTING.md
%{_sysusersdir}/group-apex.conf
%{_udevrulesdir}/70-apex.rules

%changelog
* Fri Jan 24 2025 Chris Bradbury <opensuse@chrbrd.com>
- Add `fix-for-module-import-ns.patch`.
* Sun Dec 29 2024 Chris Bradbury <opensuse@chrbrd.com>
- Add `fix-for-no_llseek.patch`.
* Wed Jul 17 2024 Chris Bradbury <opensuse@chrbrd.com>
- Fix `eventfd_signal` for kernels >= 6.8.
- Apply `fix-for-backported-dma-buf-ns.patch` to Leap 15.6 builds.
* Tue Feb 27 2024 Dominique Leuenberger <dimstar@opensuse.org>
- Use %%patch -P N instead of deprecated %%patchN.
* Tue Oct 17 2023 Chris Bradbury <chris@binaryspanner.com>
- Remove `fix-kernel-gte-6.4.patch` as it has been merged upstream.
- Fix ueficert package builds.
* Wed Jul 19 2023 Chris Bradbury <opensuse@chrbrd.com>
- Add `fix-kernel-gte-6.4.patch`.
* Thu May 11 2023 Chris Bradbury <obs@chrbrd.com>
- Add `fix-for-backported-dma-buf-ns.patch`.
  The dma-buf symbols used by this driver were moved into their own
  `DMA_BUF` module namespace in kernel 5.16. Consequently, the
    upstream source conditionally imports the `DMA_BUF` module
  namespace if the kernel version is >= 5.16.
  However, the `DMA_BUF` module namespace changes have been
  backported to the 5.14 kernel used in Leap 15.5, causing
  unpatched source builds to fail.
  This change incorporates a patch that removes the conditional
  check in the upstream source, importing the `DMA_BUF` module
  namespace regardless of kernel version, and applies that patch
  when `sle_version` is equal to "150500" (ie. is Leap 15.5 or
  greater).
  The patch is taken from the PR at:
  https://github.com/google/gasket-driver/pull/10
* Thu May 11 2023 Chris Bradbury <obs@chrbrd.com>
- Initialise package.
